/***************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (directui@nokia.com)
**
** This file is part of mhome.
**
** If you have questions regarding the use of this file, please contact
** Nokia at directui@nokia.com.
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License version 2.1 as published by the Free Software Foundation
** and appearing in the file LICENSE.LGPL included in the packaging
** of this file.
**
****************************************************************************/

#ifndef LAUNCHER_H
#define LAUNCHER_H

#include <QObject>
#include <QList>
#include <MWidgetController>
#include "launchermodel.h"
#include "applicationpackagemonitor.h"

class MDesktopEntry;
class LauncherDataStore;

/*!
 * Widget for launching and browsing installed applications.
 * The widget monitors a desktop file entry directory and creates buttons
 * that represent the applications that can be launched.
 *
 * For updating the launcher first time from launcher data store, we listen to dataStoreChanged() signal.
 * After the first initialization we disconnect from dataStoreChanged signal and rely on
 * signals for individual updates to provide the changes in desktop entries.
 *
 * Each application .desktop file must define the Name, Type and Icon keys where type is Application. The Exec key must define the application binary to be launched when the icon is selected.
 *
 * Example my_application.desktop file:
 * \code
 * [Desktop Entry]
 * Version=1.0
 * Type=Application
 * Name=my_localized_application_name
 * Comment=my_localized_application_comment
 * Exec=/usr/bin/my-app
 * Icon=my_app_icon_id
 * \endcode
 */
class Launcher : public MWidgetController
{
    Q_OBJECT
    M_CONTROLLER(Launcher)

public:
    //! A class for storing, parsing and ordering the placement information of the items
    class Placement {
    public:
        Placement();
        Placement(const QString &placement);
        Placement(int page, int position);

        void setPlacement(const QString &placementString);
        QString toString();

        QString location;
        int page;
        int position;

        bool isNull() {
            return (page < 0 || position < 0);
        }
    };

    /*!
     * Constructs a Launcher widget. The Launcher will not store/restore
     * launcher button positions and entries before a LauncherDataStore
     * has been set using setLauncherDataStore().
     *
     * \param parent parent graphics item for the widget, defaults to NULL
     * \param model model for the widget, defaults to a LauncherModel
     */
    Launcher(QGraphicsItem *parent = NULL, LauncherModel *model = NULL);

    /*!
     * Destroys the Launcher.
     */
    virtual ~Launcher();

    /*!
     * Takes a LauncherDataStore into use.
     *
     * \param dataStore LauncherDataStore for storing launcher button positions and entries
     */
    void setLauncherDataStore(LauncherDataStore *dataStore);

    /*!
     * Connects the Launcher to an ApplicationPackageMonitor for monitoring
     * installation and update progress of application packages.
     *
     * \param packageMonitor Sets a package monitor which signals package state changes
     */
    void setApplicationPackageMonitor(ApplicationPackageMonitor *packageMonitor);

    /*!
     * Sets the maximum size of the Launcher pages. Negative values are ignored.
     * If a LauncherPage already has more items than the desired maximum the
     * page is not resized.
     *
     * \param maximumPageSize the maximum number of buttons on a single LauncherPage
     */
    void setMaximumPageSize(int maximumPageSize);

    //! Returns button state enum that matches the given package state
    static LauncherButtonModel::State buttonStateFromPackageState(ApplicationPackageMonitor::PackageState);

protected:
    /*!
     * Creates a launcher button instance from a .desktop entry file.
     *
     * \param desktopEntryPath The path of the .desktop entry file to create a launcher button from
     * \return a LauncherButton representing the .desktop entry file
     */
    virtual QSharedPointer<LauncherButton> createLauncherButton(const QSharedPointer<MDesktopEntry> &desktopEntry);

    //! Creates a launcher page
    virtual QSharedPointer<LauncherPage> createLauncherPage();

    /*!
     * Returns Placement for button.
     *
     * \param desktopFileEntry is application's desktop file.
     * Name should be given in format "filename.desktop" or in absolute path.
     * Default applications directory path is used if only file name is given.
     *\return Placement object. If placement is not found for button, placement
     * is initialized with placement.page=-1 and placement.position=-1
     */
    Placement buttonPlacement(const QString &desktopEntryFile);

    /*!
     * Returns a page for adding a new launcher button.
     * Returns the last page from page list. If current last page doesn't have space for a new button,
     * then an empty page is added to the end and returned.
     *
     * \param pages The page list from where to get the page and which to update if necessary
     */
    virtual QSharedPointer<LauncherPage> pageForNewButton(QList<QSharedPointer<LauncherPage> > &pages);

    /*!
     * Remove empty pages from the given page list.
     *
     * \param pages the page list to be updated
     */
    virtual void removeEmptyPages(QList<QSharedPointer<LauncherPage> > &pages);

    /*!
     * Sets the maximum page size of a LauncherPage if defined.
     *
     * \param page the LauncherPage whose maximum size should be set
     */
    void setMaximumPageSizeIfNecessary(QSharedPointer<LauncherPage> &page);

    /*!
     * Removes button placement from the launcher data store.
     *
     * \param desktopEntryPath Desktop entry path to the desktop entry that button represents
     */
    void removeButtonPlacementFromStore(const QString &desktopEntryPath);

signals:
    /*!
     * Signal sent when a launcher button was clicked.
     */
    void launcherButtonClicked();

    /*!
     *  Signal to request panning to specific page
     *
     * \param page number of page to pan to
     */
    void panningRequested(uint page);

    /*!
     * Signal to request moving launcher focus to a page
     */
     void focusToPageRequested(uint page);

     /*!
      * Signal requesting launcher to focus to an button specified by a desktopfile.
      */
     void focusToButtonRequested(const QString& desktopFileEntry);


public slots:

    /*!
     * Requests Launcher to focus to a Launcher Button specified by a desktop file.
     *
     * \param desktopFileEntry is application's desktop file.
     * Name should be given in format "filename.desktop" or in absolute path.
     *
     * \return index of page that contains application. Returns -1 if buttons
     * is not found from launcher.
     */
    int focusToButton(const QString &desktopFileEntry);

    /*!
     * Set launcher to show a page.
     *
     *\param number of page to show.
     */
    void setPage(uint page);

    /*!
     * Updates the state of a launcher button.
     * Creates a new placeholder button if one doesn't exist for the given desktopentryfile.
     *
     * \param desktopEntry Desktop entry of the package that button represents
     * \param state State button should be set to
     * \param packageRemovable is the package represented by this button removable
     */
    void updateButtonState(const QSharedPointer<MDesktopEntry> &desktopEntry, const QString &packageName, ApplicationPackageMonitor::PackageState, bool packageRemovable);

    /*!
     * Update progress of a launcher button.
     *
     * \param desktopEntryPath Desktop entry of the package that button represents
     * \param progress Progress of operation
     * \param total Maximum progress of operation
     */
    void updateProgress(const QString& desktopEntryPath, int already, int total);

    /*!
     * Removes placeholder launcher button for an application if application is not installed.
     *
     * We can only remove button if it is found from launcher and desktop entry file paths match.
     * Different desktop entry file paths means that button is not placeholder and hence needs to be removed by
     * launcher data store.
     *
     * \param desktopEntryPath of an application
     */
    void removePlaceholderButton(const QString &desktopEntryPath);
    /*!
     * Updates current Launcher Button placements on LauncherPage to desktop files.
     *
     * \param Page for update
     */
    void updateButtonPlacementsOnPage(LauncherPage *page);

    /*!
     * Remove a launcher button from launcher.
     *
     * \param desktopEntryPath Path to desktop entry that button to be removed represents
     */
    virtual void removeLauncherButton(const QString &desktopEntryPath);

protected slots:
    /*!
     * Updates pages according to the contents of the data store.
     */
    virtual void updatePagesFromDataStore();

private slots:
    /*!
     * Add a launcher button to launcher.
     *
     * If button already exists in launcher then it is updated.
     *
     * \param desktopEntry Desktop entry for the button
     */
    void addLauncherButton(const QSharedPointer<MDesktopEntry> &desktopEntry);

    /*!
     * Updates a launcher button from a .desktop entry file.
     *
     * \param desktopEntry Desktop entry for the button
     *
     * \return Whether button for the desktop entry was found
     */
    bool updateLauncherButton(const QSharedPointer<MDesktopEntry> &desktopEntry);

    /*!
     * Cleans up the page that sent a signal to this slot after button removal. Removes the
     * page if it became empty, updates button placements otherwise.
     *
     * \param removedButton Button to be removed while pruning. No buttons removed if NULL.
     */
    void prunePage(const QString &removedEntry = QString());

private:
    /*!
     * Get desktop entry placement in datastore
     *
     * \param desktopEntryPath Path to desktop entry
     * \return Placement of the entry, if entry is not in the datastore or the datastore is not set then null placement is returned
     */
    Launcher::Placement entryPlacementInDatastore(const QString &desktopEntryPath);

    /*!
     * Find the button corresponding to the desktop entry path.
     *
     * \param desktopEntryPath path to the desktop entry of the button
     * \return pointer to the button, or null if the button was not found
     */
    QSharedPointer<LauncherButton> findButtonForDesktopEntryPath(const QString &desktopEntryPath);

    /*!
     * Appends given launcher button to given pages.
     *
     * Finds first available space from last page or adds a new page if last page doesn't have space.
     *
     * \param button Button to be appended
     * \param pages List of pages where button is to be appended
     * \return placement where button was added
     */
    Launcher::Placement appendButtonToPages(QSharedPointer<LauncherButton> button, QList<QSharedPointer<LauncherPage> > &pages);

    /*!
     * Update the given pages list by putting the desktop entries with known
     * placements in the desired pages. Pages are created as necessary.
     * There may be empty pages in the page list after calling this.
     *
     * \param pages the page list to be updated
     */
    void addDesktopEntriesWithKnownPlacements(QList<QSharedPointer<LauncherPage> > &pages);

    /*!
     * Update the given pages list by putting the desktop entries with no known
     * placements on the last page. Pages are created as necessary.
     *
     * \param pages the page list to be updated
     */
    void addDesktopEntriesWithUnknownPlacements(QList<QSharedPointer<LauncherPage> > &pages);

    /*!
     * Creates a map that contains the placement of each given desktop entry in the launcher.
     * Only the items that have a recognized placement are included in the map.
     * The placement is recognized when it is in the launcher/page/position format
     * (for example "launcher/1/3").
     *
     * \return map containing the placement of each given desktop entry in the launcher
     */
    QMap<Launcher::Placement, QString> createPlacementMap(const QHash<QString, QVariant> &desktopEntryPlacements);

    /*!
     * Gets a placeholder button for given entry.
     * Finds and returns a button that represents the given entry. If button is not yet found in launcher
     * we add a new placeholder button.
     *
     * \param desktopEntry Desktop entry the button should represent
     */
    QSharedPointer<LauncherButton> placeholderButton(const QSharedPointer<MDesktopEntry> &desktopEntry);

    /*!
     * If the button in the given desktop entry path has a corresponding installer-extra desktop entry,
     * makes the button's desktop entry point to that path.
     *
     * \param desktopEntryPath path to the desktop entry of the button
     * \return true if the conversion could be done, false otherwise
     */
    bool convertToPlaceHolderButton(const QString &desktopEntryPath);

    /*!
     * Updates button placement to the launcher data store.
     *
     * If button is not already in store, a new value is created.
     *
     * \param desktopEntryPath Desktop entry path to the desktop entry that button represents
     */
    void updateButtonPlacementInStore(const QString &desktopEntryPath);

    //! Returns index of given page
    int pageIndex(LauncherPage *page);

    //! A string used for identifying content to be placed in the launcher
    static const QString LOCATION_IDENTIFIER;

    //! Separator character for the launcher data store values
    static const char SECTION_SEPARATOR;

    //! A template for the launcher content placement string in the data store
    static const QString PLACEMENT_TEMPLATE;

    //! LauncherDataStore for storing launcher button positions and entries
    LauncherDataStore *dataStore;

    //! packageMonitor which signals the package state changes
    ApplicationPackageMonitor *packageMonitor;

    //! The maximum page size of each LauncherPage
    int maximumPageSize;

    //! Whether the launcher has been initialized or not
    bool initialized;

    //! This list contains application desktop entries of applications which are currently uninstalling
    QStringList uninstalling;

#ifdef UNIT_TEST
    friend class Ut_Launcher;
#endif
};

#endif /* LAUNCHER_H */
